jQuery.sap.declare("sas.stress.wkbench.widget.SlimsheetSource");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.model.Project");
jQuery.sap.require("sas.stress.wkbench.model.TreeNode");
//jQuery.sap.require("sas.hc.ui.commons.Callout");

sas.stress.wkbench.widget.SlimsheetSource = function (sheetId, filterId/*scenarioId*/) {
    this.sheetId = sheetId;
    this.filterId/*scenarioId*/ = filterId;
    this.widths = null;
    this.info = undefined;
    this.selectedCol=0;
    this.selectedRow=0;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.init = function () {
    var project = sas.stress.wkbench.model.Project.getCurrentProject();
    var sheet = project.getSheet(this.sheetId);
    this.sheetName=sheet.getName();
    var info = sheet.getInfo();
    //this.scenario=info.getScenario(this.scenarioId);
    this.sheetFilters = info.getFilters();
    this.data = this.sheetFilters.getData();
    this.infoData = info.getData();
    this.projectCreatedTime = project.getCreatedTime();
};
/*sas.stress.wkbench.widget.SlimsheetSource.prototype.getCellScenario=function(row,col){
 return this.scenario;
 };*/
sas.stress.wkbench.widget.SlimsheetSource.prototype.getCellFilter = function (row, col) {
    return this.sheetFilters;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getInfo = function () {
    if (this.info){
        return this.info;
    }
    // we are coming in again... check the local storage for widths
    var stored=this.getFromLocalStorage()||null;
    if (stored){
        this.widths=[];
        this.widths[0]=stored.fixed.widths;
        this.widths[1]=stored.scrolled.widths;
    }

    var that=this;
    var i, col;
    var columns = this.infoData.columns;
    var headerCount = (this.infoData.columnHeadersCount > 0) ? this.infoData.columnHeadersCount + 1 : 2;
    var rowHierarchyCount = (this.infoData.rowHierarchiesCount > 0) ? this.infoData.rowHierarchiesCount + 1 : 2;

    // we are coming in again... check the local storage for widths
    var stored=this.getFromLocalStorage()||null;
    if (stored&&(stored.fixed.widths.length===rowHierarchyCount||stored.scrolled.widths.length===headerCount)){ // something needs to match correctly
        this.widths=[];
        this.widths[0]=stored.fixed.widths;
        this.widths[1]=stored.scrolled.widths;
    }else{
        // delete the user preference... it has wrong data
        var project=this.sheetFilters.getSheetInfo().getSheet().getProject();
        var sheetDefn= _.find(project.getSheetDefinitions(),function(sheetDefinition){
            return sheetDefinition.sheetTemplateName.toLowerCase()===that.sheetName.toLowerCase();
        });
        if (sheetDefn) {
            sas.stress.wkbench.UserPrefs.delete("sas.stress.wkbench.sheet.template." + sheetDefn.sheetTemplateKey + ".columns.widths");
        }
    }

    var eachColumnWidth=30;
    var lastColumnWidth=(rowHierarchyCount-1)*eachColumnWidth+200;
    if (!this.widths) {//default column widths
        this.widths = [];
        if (rowHierarchyCount === 2) {
            this.widths[0] = [50, 100];   // there can be this case and therefore, the width is now reduced to a minimum
        } else {
            this.widths[0] = [];
            this.widths[0].push(50);
            for (var idx = 1; idx < rowHierarchyCount-1; idx++) {
                this.widths[0].push(eachColumnWidth);	// fixes for S1230285
            }
            this.widths[0].push(lastColumnWidth);	// fixes for S1230285
        }
        this.widths[1] = [/*162*/];
        for (i = 0; i < columns.length; i++) {
            this.widths[1].push(120);
        }
    }
    var fixedWidth = 0/*this.widths[0][0] + this.widths[0][1]*/;
    /*if (rowHierarchyCount > 2) {
     fixedWidth = 0;
     for (var idx = 0; idx < rowHierarchyCount; idx++) {
     fixedWidth += this.widths[0][idx];
     }
     }*/
    var scrolledWidth = 0;
    var labelname = [];
    var rowlabel = [];
    for (var idx = 0; idx < headerCount; idx++) {
        labelname[idx] = "label_" + idx;
    }
    for (var idx = 0; idx < rowHierarchyCount; idx++) {
        rowlabel[idx] = "label_" + idx;
    }

    for (i = 0; i < this.widths[1].length; i++) {
        scrolledWidth += this.widths[1][i];
    }
    this.info={
        rowCount: this.getRowCount(),
        headerCount: headerCount,
        groupCount: 1,
        fixed: {
            widths: this.widths[0],
            width: fixedWidth,
            colCount: rowHierarchyCount,
            rowCount: this.infoData.rowLength
        },
        scrolled: {
            widths: this.widths[1],
            width: scrolledWidth,
            colCount: columns.length/*+1*/
        }
    };
    var columnheadersmap, rowheadersmap;
    if (this.infoData._getColumnHeadersMap()) {
        columnheadersmap = this.infoData._getColumnHeadersMap();
    }
    if (this.infoData._getRowHeadersMap()) {
        rowheadersmap = this.infoData._getRowHeadersMap();
    }

    for (var idx = 0; idx < headerCount; idx++) {
        this.info.scrolled[labelname[idx]]=columnheadersmap?[]:[""];
    }
    for (var idx = 0; idx < rowHierarchyCount; idx++) {
        this.info.fixed[rowlabel[idx]]=rowheadersmap?undefined:["",""];
        if (idx === 0 && rowheadersmap) {
            this.info.fixed[rowlabel[idx]]=[""];
        }
    }

    for (var i = 0; i < columns.length; i++) {
        var col = columns[i];
        var j = 1;
        this.info.scrolled[labelname[0]].push(col.short);
        while (j < headerCount/*-1*/) {
            if (columnheadersmap) {
                var data = { nodes: [], label: [/*""*/] };
                var nodes = columnheadersmap["headerrow_" + (j - 1)] || [];
                for (var idx = 0; idx < nodes.length; ++idx) {
                    var node = nodes[idx];
                    var span = sas.stress.wkbench.model.TreeNode.getSpan(node);
                    for (var spnidx = 0; spnidx < span; spnidx++) {
                        data.nodes.push(node);
                        data.label.push((j === (headerCount - 1) ? node.getDataKey() : node.getName()));
                    }
                }
                this.info.scrolled[labelname[j]]=data;
            } else {
                this.info.scrolled[labelname[j]].push(col?col.long:"");
            }
            j++;
        }
    }
    var j = 1;
    var textWidth,prevWidth=0;
    while (j < rowHierarchyCount) {
        var data = {nodes: [], label: []}, parent;
        if (rowheadersmap) {
            data = { nodes: [], label: [] };
            var nodes = rowheadersmap["headerrow_" + (j - 1)] || [];
            if (nodes) {
                for (var idx = 0; idx < nodes.length; ++idx) {
                    var node = nodes[idx];
                    var span = sas.stress.wkbench.model.TreeNode.getSpan(node);
                    for (var spnidx = 0; spnidx < span; spnidx++) {
                        data.nodes.push(node);
                        data.label.push(node.getName());
                    }
                }

                this.info.fixed[rowlabel[j]] = data;
            } else {
                this.info.fixed[rowlabel[j]] = data;
            }
            if (j===(rowHierarchyCount-1)) {
                var that=this;
                _.each(data.label,function(txt){
                    if (""!==txt) {
                        textWidth = that.getWidthOfText(txt, "'AvenirNext', Helvetica, arial,sans,sans-serif", "13px"); // FIXME: should not hard-coded font info
                        if (prevWidth===0){
                            prevWidth=textWidth;
                        }else{
                            prevWidth=Math.max(prevWidth,textWidth);
                        }
                    }
                });
            }
        } else {
            this.info.scrolled[rowlabel[j]].push("");
        }
        j++;
    }
    this.info.fixed.widths[j-1]=Math.min(this.info.fixed.widths[j-1],prevWidth);    // give a spacing of (10-5)px at the end. the start will take 5
    fixedWidth = this.widths[0][0] + this.widths[0][1];
    if (rowHierarchyCount > 2) {
        fixedWidth = 0;
        for (var idx = 0; idx < rowHierarchyCount; idx++) {
            fixedWidth += this.widths[0][idx];
        }
    }
    this.info.fixed.width=fixedWidth;

    return this.info;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getRowCount = function () {
    return this.infoData.rows.length;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getRowHeadersCount = function () {
    return this.infoData.rowHierarchiesCount;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getColumnHeadersCount = function () {
    return this.infoData.columnHeadersCount;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getColumnCount = function () {
    return this.infoData.columns.length;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getFixedRowData = function (rowNum) {
    var rowInfo = this.infoData.rows[rowNum];
    var tab = "";
    for (var i = 0; i < rowInfo.level; i++) {
        tab += "     ";
    }
    return [(rowNum + 1).toString(), tab + rowInfo.name];
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getScrolledRowData = function (rowNum) {
    var rowInfo = this.infoData.rows[rowNum];
    var row = this.data.data[rowNum];
    /*var data=[rowInfo.long];
     return data.concat(row);*/
    return row;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.isSelectable = function (row, col) {
    return !isNaN(row) && !isNaN(col) /*&& col!==0*/ && this.infoData.rows[row].long !== null;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.decorateCells = function (control) {
    var i, j, overlay, isOverriden, isFormulaic,prevClass;
    var cell;
    var that = this;
    for (i in this.data.ncFormulas) {
        overlay = this.data.ncFormulas[i];
        isOverriden = (!overlay.timestamp || (overlay.timestamp > this.projectCreatedTime)) ? " SlimOverriden" : "";
        isFormulaic = overlay.expr.type !== "NAME" ? " SlimFormulaic" : "";
        if (isOverriden || isFormulaic) {
            var cell = sas.stress.wkbench.widget.Slimsheet.findCell(control, overlay.rowIndex, overlay.columnIndex/*+1*/);
            if (cell) {
                prevClass  = cell.className;
                cell.className = prevClass ? prevClass + isOverriden + isFormulaic : isOverriden + isFormulaic;
            }
        }
    }
    var callout;
    var processed = [];
    // create just one control for all the cells...
    // we will remove the existing content and add the new content with new data everytime the cell reference is changed
    if (_.size(this.data.ncComments)>0||_.size(this.data.cComments)>0||_.size(this.data.cValidationErrors)>0) {
        callout = new sap.m.ResponsivePopover({
            showHeader: false,
            verticalScrolling: false,
            placement: sap.m.PlacementType.Top,
            modal: false,
            offsetY: 10
        });
        /*callout.onmouseover = function (e) {
         console.error("ONMOUSEOVER");
         cell = e.currentTarget;
         callout.openBy(cell);
         // force the focus into the responseive popver.. it will handle the first "focus"able item
         //callout.focus();
         };*/
        callout.addStyleClass("stressCommentPopover");
    }
    for (i in this.data.cDrillDowns){
        overlay=this.data.cDrillDowns[i];
        if (overlay.propertyKey) {
            cell = sas.stress.wkbench.widget.Slimsheet.findCell(control, overlay.rowIndex, overlay.columnIndex);
            if (cell) {
                prevClass = cell.className;
                cell.className = prevClass ? prevClass + " SlimDrillDownAble" : "SlimDrillDownAble";
            }

            // if there are stored widths.. this will occur if the user resized the columns
            var parsedWidths = this.getFromLocalStorage();
            if (parsedWidths) {
                // we will set up the right on the drill-down CSS to make sure the decorator comes at the right place
                $(".SlimDrillDownAble>div.SlimDrillDownAbleDiv").css("right", parsedWidths["scrolled"].widths[overlay.columnIndex] - 14 + "px");
            }
        }
    }
    for (i in this.data.ncComments) {
        overlay = this.data.ncComments[i];
        cell = sas.stress.wkbench.widget.Slimsheet.findCell(control, overlay.rowIndex, overlay.columnIndex/*+1*/);
        if (cell) {
            prevClass = cell.className;
            cell.className = prevClass ? prevClass + " SlimCommented" : "SlimCommented";
            var validationoverlaykey = "$" + overlay.columnIndex + "$" + overlay.rowIndex;
            var validationoverlay = this.data.cValidationErrors[validationoverlaykey];
            if (validationoverlay) {
                prevClass = cell.className;
                cell.className = prevClass ? prevClass + " SlimValidationError" : "SlimValidationError";
                processed.push(validationoverlaykey);
            }
            //callout = new sap.ui.commons.Callout({offsetY:12});
            cell.onmouseover = (function (overlay, filter, callout, cell) {
                return function () {
                    var $cell = $(cell);
                    var cellRef = new sas.stress.wkbench.model.CellRef(overlay.columnIndex, overlay.rowIndex, filter/*scenario*/);
                    validationoverlaykey = "$" + cellRef.column + "$" + cellRef.row;
                    validationoverlay = that.data.cValidationErrors[validationoverlaykey];
                    var cellCommentEditor = sas.stress.wkbench.ViewManager.loadCellCommentEditor(cellRef, true, validationoverlay !== undefined);
                    cellCommentEditor.attachClosing(function () {
                        this.destroy(); // destroy the view rather than keeping it around, causes other issues
                    }, callout);
                    callout.removeAllContent(); // remove existing content
                    callout.addContent(cellCommentEditor);  // add the new content
                    callout.setInitialFocus(cellCommentEditor.commentTextArea); // initial focussed item
                    callout._currentControl = {
                        $: function () {
                            return $cell;
                        },
                        getDomRef: function () {
                            return cell;
                        },
                        getFocusInfo: function () {
                            return { id: control.id };
                        },
                        applyFocusInfo: function (obj) {
                            $("#" + obj.id).focus();
                        },
                        getTooltip: function () {
                            return callout;
                        }
                    };
                    //return callout.onmouseover.apply(callout, arguments);
                    return callout.openBy(cell);
                };
            })(overlay, this.sheetFilters, callout, cell);
            cell.onmouseout = (function (callout) {
                return function (e) {
                    //callout.onmouseout.apply(callout, arguments);
                    console.error("Cell onmouseout");
                    if (!e.ctrlKey) {
                        var calldom = callout.getDomRef();
                        var evt = $.Event('keydown');
                        evt.which = 27;
                        evt.altKey = false;
                        evt.keyCode = 27;
                        $(calldom).trigger(evt);
                    }
                };
            })(callout);
        }
    }
    for (i in this.data.cValidationErrors) {
        if (processed.indexOf(i, 0) === -1) {	// not processed
            overlay = this.data.cValidationErrors[i];
            cell = sas.stress.wkbench.widget.Slimsheet.findCell(control, overlay.rowIndex, overlay.columnIndex/*+1*/);
            if (cell) {
                prevClass = cell.className;
                cell.className = prevClass ? prevClass + " SlimValidationError" : "SlimValidationError";
                //callout = new sas.hc.ui.commons.Callout();
                cell.onmouseover = (function (overlay, filter/*scenario*/, callout, cell) {
                    return function () {
                        var $cell = $(cell);
                        var cellRef = new sas.stress.wkbench.model.CellRef(overlay.columnIndex, overlay.rowIndex, filter/*scenario*/);
                        var cellCommentEditor = sas.stress.wkbench.ViewManager.loadCellCommentEditor(cellRef, false, true);
                        cellCommentEditor.attachClosing(function () {
                            this.destroy();
                        }, callout);
                        callout.removeAllContent();
                        callout.addContent(cellCommentEditor);
                        //callout.setInitialFocus(cellCommentEditor.commentTextArea);
                        callout._currentControl = {
                            $: function () {
                                return $cell;
                            },
                            getDomRef: function () {
                                return cell;
                            },
                            getFocusInfo: function () {
                                return { id: control.id };
                            },
                            applyFocusInfo: function (obj) {
                                $("#" + obj.id).focus();
                            },
                            getTooltip: function () {
                                return callout;
                            }
                        };
                        //return callout.onmouseover.apply(callout, arguments);
                        return callout.openBy(cell);
                    };
                })(overlay, this.sheetFilters/*.scenario*/, callout, cell);
                cell.onmouseout = (function (callout) {
                    return function () {
                        //callout.onmouseout.apply(callout, arguments);
                        console.error("Cell onmouseout");
                        var calldom=callout.getDomRef();
                        var evt=$.Event('keydown');
                        evt.which=27;
                        evt.altKey=false;
                        evt.keyCode=27;
                        $(calldom).trigger(evt);
                    };
                })(callout);
            }
        }
    }

    var cells;
    for (i = 0; i < this.infoData.rowsLocked.length; i++) {
        cells = sas.stress.wkbench.widget.Slimsheet.findRow(control, this.infoData.rowsLocked[i].index);
        for (j = 0; j < cells.length; j++) {
            cell = cells[j];
            if (cell) {
                prevClass = cell.className;
                cell.className = prevClass ? prevClass + " SlimLocked" : "SlimLocked";
            }
        }
    }
    for (i = 0; i < this.infoData.rowsRoots.length; i++) {
        cell = sas.stress.wkbench.widget.Slimsheet.findCell(control, this.infoData.rowsRoots[i].index, 1, true);
        if (cell) {
            prevClass = cell.className;
            cell.className = prevClass ? prevClass + " SlimHierarchyRoot" : "SlimHierarchyRoot";
        }
    }
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.augmentCell = function (control,dims,source,id,row,col,i,j,inRange,values,isScrolled,td,span,div,text) {
    if (isScrolled) {
        /*if(col===0){
         td.className="SlimStart";
         }else*/
        {
            var currentProject = sas.stress.wkbench.model.Project.getCurrentProject();
            if (this.infoData.columns[col].readonly || currentProject.isLocked()) {//history columns
                td.className = "SlimNotEditable";
            }
            var innerdiv = document.createElement("DIV");
            innerdiv.className = "SlimFormulaicDiv";
            div.appendChild(innerdiv);
            innerdiv = document.createElement("DIV");
            innerdiv.className = "SlimCommentedDiv";
            div.appendChild(innerdiv);
            innerdiv = document.createElement("DIV");
            innerdiv.className = "SlimValidationErrorDiv";
            div.appendChild(innerdiv);

            innerdiv = document.createElement("DIV");
            innerdiv.className = "SlimDrillDownAbleDiv";
            div.appendChild(innerdiv);
        }
    }
    return td;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getFormatter = function (row) {
    var rowInfo = this.infoData.rows[row];
    return rowInfo && rowInfo.formatter;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getRealColumn = function (column) {
    return column;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getCalculatedColumn = function (col) {
    return col;
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.setSelectedColumn = function (col) {
    this.selectedCol = col;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getSelectedColumn = function () {
    return this.selectedCol;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.setSelectedRow = function (row) {
    this.selectedRow = row;
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getSelectedRow = function () {
    return this.selectedRow;
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.putToLocalStorage=function(){
    var that=this;
    var stored={fixed:{width:0, widths:[]}, scrolled:{width:0, widths:[]}};
    stored.fixed.width=this.info.fixed.width;
    stored.fixed.widths=this.info.fixed.widths;

    stored.scrolled.width=this.info.scrolled.width;
    stored.scrolled.widths=this.info.scrolled.widths;
    var project=this.sheetFilters.getSheetInfo().getSheet().getProject();
    var sheetDefn= _.find(project.getSheetDefinitions(),function(sheetDefinition){
        return sheetDefinition.sheetTemplateName.toLowerCase()===that.sheetName.toLowerCase();
    });
    if (sheetDefn) {
        sas.stress.wkbench.UserPrefs.save("sas.stress.wkbench.sheet.template." + sheetDefn.sheetTemplateKey + ".columns.widths", JSON.stringify(stored), project.getId());
    }
    //localStorage[this.sheetName]=JSON.stringify(stored);
};
sas.stress.wkbench.widget.SlimsheetSource.prototype.getFromLocalStorage=function(){
    var that=this;
    var project=this.sheetFilters.getSheetInfo().getSheet().getProject();
    var sheetDefn= _.find(project.getSheetDefinitions(),function(sheetDefinition){
        return sheetDefinition.sheetTemplateName.toLowerCase()===that.sheetName.toLowerCase();
    });
    if (sheetDefn) {
        var stored = sas.stress.wkbench.UserPrefs.get("sas.stress.wkbench.sheet.template." + sheetDefn.sheetTemplateKey + ".columns.widths",project.getId());
        //var stored= localStorage[this.sheetName];
        if (stored) {
            return JSON.parse(stored);
        }
    }
    return undefined;
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.clearInfo=function(){
    this.info={};
    this.info=undefined;
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.getWidthOfText = function(txt, fontname, fontsize){
    // Create dummy span
    this.e = document.createElement('span');
    // Set font-size
    this.e.style.fontSize = fontsize;
    // Set font-face / font-family
    this.e.style.fontFamily = fontname;
    // Set text
    this.e.innerHTML = txt;
    document.body.appendChild(this.e);
    // Get width NOW, since the dummy span is about to be removed from the document
    var w = this.e.offsetWidth;
    // Cleanup
    document.body.removeChild(this.e);
    // All right, we're done
    return w;
};

sas.stress.wkbench.widget.SlimsheetSource.prototype.destroy=function(){
    this.widths=[];
    this.widths=null;
    if (this.infoData){
        this.infoData.destroy();
    }
    if (this.sheetFilters){
        this.sheetFilters.destroy();
    }

    var keys=[],idx,i;
    if (this.info){
        var keys= _.keys(this.info.fixed);
        for (idx=keys.length-1;idx>=0;--idx){
            var data=this.info.fixed[keys[idx]];
            if (data&&data.nodes) {
                for (i = data.nodes.length - 1; i >= 0; --i) {
                    var node = data.nodes[i];
                    node.destroy();
                    node = undefined;
                    delete node;
                }
                data.nodes.splice(0);
                delete data.nodes;
                data.label.splice(0);
                delete data.label;
                delete data;
            }
        }
        keys=[];
        keys= _.keys(this.info.scrolled);
        for (idx = keys.length-1; idx >= 0; --idx) {
            data = this.info.scrolled[keys[idx]];
            if (data&&data.nodes) {
                for (i = data.nodes.length - 1; i >= 0; --i) {
                    var node = data.nodes[i];
                    node.destroy();
                    node = undefined;
                    delete node;
                }
                data.nodes.splice(0);
                delete data.nodes;
                data.label.splice(0);
                delete data.label;
                delete data;
            }
        }
        keys=[];
    }
    this.info={};
    this.info=undefined;
    delete this.info;
    this.infoData=undefined;
    delete this.infoData;
    this.sheetFilters=undefined;
    delete this.sheetFilters;
    this.data=undefined;
    delete this.data;
};