%let sashome_dir = %sysget(SASHOME);
%let as_of_date = 2016-08-01;

options
   metaserver = "<metadata_server>"
   metaport   = <port>
   metauser   = <metadata_user>
   metapass   = <password>
   ;

filename LUAPATH ( '!SASROOT/misc/stresssvr/lua/sas' '!SASROOT/stresssvr/sasmisc/lua/sas' );

%let log_level = 1;


proc lua restart;
submit;

   local rest       = require 'risk.stress.rest_service'
   local json       = require 'risk.stress.json'
   local metadata   = require 'risk.stress.metadata'
   local rfw        = require 'risk.stress.rfw_api'
   local utils      = require 'risk.stress.rest_utils'
   local file       = require 'risk.stress.file_utils'
   local content    = require 'risk.stress.rfw_content_utils'

   json.strictTypes = true
   local file_sep = utils.get_file_sep()

   local samp_dir = sas.symget('sashome_dir')
   if sas.index("WIN", sas.symget('sysscp')) > 0 then
      samp_dir = samp_dir..'/SASFoundation/9.4/stresssvr/sasmisc/sample_code/drilldown_sample'
   else
      samp_dir = samp_dir..'/SASFoundation/9.4/misc/stresssvr/sample_code/drilldown_sample'
   end
   
   local as_of_date  = sas.symget('as_of_date')

   -- First get the rest api information from metadata
   local hostname, port, protocol, appName, URL = metadata.get_rest_end_point_dtls("SASRiskandFinanceWorkbench REST")
   if not URL then
      sas.print("%1z Was unable to get the metadata information - check that the server is running and that you have the proper permissions")
      return
   end
   sas.print("%3z Got the following information from metadata:")
   sas.print("  Hostname: "..hostname)
   sas.print("  Port: "..port)
   sas.print("  Protocol: "..protocol)
   sas.print("  Application Name: "..appName)
   sas.print("  URL: "..URL)

   -- Set the url for rest calls
   local pass = rest.set_url(URL)
   if not pass then sas.print("%1zUnable to set the rest url.") return end

   ------------------------------------------------------------------------------
   --                            UPLOAD HIERARCHIES                            --
   ------------------------------------------------------------------------------
   --Drilldown
   local hier_list = utils.read_file(samp_dir..file_sep..'hierarchies'..file_sep..'drilldown.txt')
   hier_list = string.split(hier_list, '\n')
   for i, filename in ipairs(hier_list) do
      local hier_name = sas.tranwrd(filename,'.xlsx','')
      hier_name = string.trim(hier_name)
      filename = string.gsub(filename, "\n", "")
      filename = string.trim(filename)
      sas.print("%3zCreating Hierarchy "..hier_name.." in category Drillldown from the file "..samp_dir..file_sep..'Hierarchy'..file_sep..filename)
      local hpass, hout_str, hcode = rfw.hierarchy_create('Drilldown', hier_name, "For drilldown sample project",samp_dir..file_sep..'hierarchies'..file_sep..filename)
      if not hpass then
         sas.print("%1z "..(utils.read_fileref('out') or ""))
         return
      end
   end

    --Time
   local hier_list = utils.read_file(samp_dir..file_sep..'hierarchies'..file_sep..'time.txt')
   hier_list = string.split(hier_list, '\n')
   for i, filename in ipairs(hier_list) do
      hier_name = sas.tranwrd(filename,'.xlsx','')
      hier_name = string.trim(hier_name)
      filename = string.gsub(filename, "\n", "")
      filename = string.trim(filename)
      sas.print("%3zCreating Hierarchy "..hier_name.." in category Time from the file "..samp_dir..file_sep..'Hierarchy'..file_sep..filename)
      local hpass, hout_str, hcode = rfw.hierarchy_create('Time', hier_name, "For drilldown sample project",samp_dir..file_sep..'hierarchies'..file_sep..filename)
      if not hpass then
         sas.print("%1z "..(utils.read_fileref('out') or ""))
         return
      end      
   end
   --Entity
   local hier_list = utils.read_file(samp_dir..file_sep..'hierarchies'..file_sep..'entity.txt')
   hier_list = string.split(hier_list, '\n')
   for i, filename in ipairs(hier_list) do
      hier_name = sas.tranwrd(filename,'.xlsx','')
      hier_name = string.trim(hier_name)
      filename = string.gsub(filename, "\n", "")
      filename = string.trim(filename)
      sas.print("%3zCreating Hierarchy "..hier_name.." in category Entity from the file "..samp_dir..file_sep..'Hierarchy'..file_sep..filename)
      local hpass, hout_str, hcode = rfw.hierarchy_create('Entity', hier_name, "For drilldown sample project",samp_dir..file_sep..'hierarchies'..file_sep..filename)
      if not hpass then   
         sas.print("%1z "..(utils.read_fileref('out') or ""))
         return
      end      
   end 

   ------------------------------------------------------------------------------
   --                                   SCRIPTS                                --
   ------------------------------------------------------------------------------
   sas.print("%3z Loading the Scripts")
   content.script_load_from_file(samp_dir..file_sep..'scripts'..file_sep..'Scripts.xlsx')


   ------------------------------------------------------------------------------
   --                                MODEL GROUPS                              --
   ------------------------------------------------------------------------------
   sas.print("%3z Loading the Model Groups")
   content.model_group_load_from_file(samp_dir..file_sep..'modelgroups'..file_sep..'Model_Groups.xlsx')


   ------------------------------------------------------------------------------
   --                                 TEMLPATES                                --
   ------------------------------------------------------------------------------
   sas.print("%3z Loading the worksheet templates")
   content.worksheet_batch_load(samp_dir..file_sep..'worksheets')


   ------------------------------------------------------------------------------
   --                                PROCESSES                                 --
   ------------------------------------------------------------------------------
   sas.print("%3zCreating the sample drilldown process")
   rfw.process_create(samp_dir..file_sep.."process"..file_sep.."Drilldown_Sample.xlsx")


   ------------------------------------------------------------------------------
   --                                PROJECTS                                  --
   ------------------------------------------------------------------------------
   sas.print("%3zCreating the Drilldown Sample Project")
   rfw.project_create("Drilldown Sample", "Sample project for using drilldown with detail data", "Drilldown_Sample", as_of_date,'0',{[1] = "summary"})


endsubmit;
run;
